@@ -14,7 +14,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $OpenBSD: krl.c,v 1.13 2013/07/20 22:20:42 djm Exp $ */
+/* $OpenBSD: krl.c,v 1.14 2014/01/31 16:39:19 tedu Exp $ */
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -236,7 +236,7 @@ insert_serial_range(struct revoked_serial_tree *rt, u_int64_t lo, u_int64_t hi)
 	struct revoked_serial rs, *ers, *crs, *irs;
 
 	KRL_DBG(("%s: insert %llu:%llu", __func__, lo, hi));
-	bzero(&rs, sizeof(rs));
+	memset(&rs, 0, sizeof(rs));
 	rs.lo = lo;
 	rs.hi = hi;
 	ers = RB_NFIND(revoked_serial_tree, rt, &rs);
@@ -1113,7 +1113,7 @@ is_key_revoked(struct ssh_krl *krl, const Key *key)
 	struct revoked_certs *rc;
 
 	/* Check explicitly revoked hashes first */
-	bzero(&rb, sizeof(rb));
+	memset(&rb, 0, sizeof(rb));
 	if ((rb.blob = key_fingerprint_raw(key, SSH_FP_SHA1, &rb.len)) == NULL)
 		return -1;
 	erb = RB_FIND(revoked_blob_tree, &krl->revoked_sha1s, &rb);
@@ -1124,7 +1124,7 @@ is_key_revoked(struct ssh_krl *krl, const Key *key)
 	}
 
 	/* Next, explicit keys */
-	bzero(&rb, sizeof(rb));
+	memset(&rb, 0, sizeof(rb));
 	if (plain_key_blob(key, &rb.blob, &rb.len) != 0)
 		return -1;
 	erb = RB_FIND(revoked_blob_tree, &krl->revoked_keys, &rb);
@@ -1145,7 +1145,7 @@ is_key_revoked(struct ssh_krl *krl, const Key *key)
 		return 0; /* No entry for this CA */
 
 	/* Check revocation by cert key ID */
-	bzero(&rki, sizeof(rki));
+	memset(&rki, 0, sizeof(rki));
 	rki.key_id = key->cert->key_id;
 	erki = RB_FIND(revoked_key_id_tree, &rc->revoked_key_ids, &rki);
 	if (erki != NULL) {
@@ -1160,7 +1160,7 @@ is_key_revoked(struct ssh_krl *krl, const Key *key)
 	if (key_cert_is_legacy(key) || key->cert->serial == 0)
 		return 0;
 
-	bzero(&rs, sizeof(rs));
+	memset(&rs, 0, sizeof(rs));
 	rs.lo = rs.hi = key->cert->serial;
 	ers = RB_FIND(revoked_serial_tree, &rc->revoked_serials, &rs);
 	if (ers != NULL) {
