@@ -1,8 +1,9 @@
-#	$OpenBSD: proxy-connect.sh,v 1.5 2002/12/09 15:28:46 markus Exp $
+#	$OpenBSD: proxy-connect.sh,v 1.6 2013/03/07 00:20:34 djm Exp $
 #	Placed in the Public Domain.
 
 tid="proxy connect"
 
+verbose "plain username"
 for p in 1 2; do
 	${SSH} -$p -F $OBJ/ssh_proxy 999.999.999.999 true
 	if [ $? -ne 0 ]; then
@@ -16,3 +17,10 @@ for p in 1 2; do
 		fail "bad SSH_CONNECTION"
 	fi
 done
+
+verbose "username with style"
+for p in 1 2; do
+	${SSH} -$p -F $OBJ/ssh_proxy ${USER}:style@999.999.999.999 true || \
+		fail "ssh proxyconnect protocol $p failed"
+done
+
