@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh-add.c,v 1.108 2013/12/19 00:10:30 djm Exp $ */
+/* $OpenBSD: ssh-add.c,v 1.109 2014/02/02 03:44:31 djm Exp $ */
 /*
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
@@ -84,7 +84,7 @@ static void
 clear_pass(void)
 {
 	if (pass) {
-		memset(pass, 0, strlen(pass));
+		explicit_bzero(pass, strlen(pass));
 		free(pass);
 		pass = NULL;
 	}
@@ -360,7 +360,7 @@ lock_agent(AuthenticationConnection *ac, int lock)
 			fprintf(stderr, "Passwords do not match.\n");
 			passok = 0;
 		}
-		memset(p2, 0, strlen(p2));
+		explicit_bzero(p2, strlen(p2));
 		free(p2);
 	}
 	if (passok && ssh_lock_agent(ac, lock, p1)) {
@@ -368,7 +368,7 @@ lock_agent(AuthenticationConnection *ac, int lock)
 		ret = 0;
 	} else
 		fprintf(stderr, "Failed to %slock agent.\n", lock ? "" : "un");
-	memset(p1, 0, strlen(p1));
+	explicit_bzero(p1, strlen(p1));
 	free(p1);
 	return (ret);
 }
